@@ -35,6 +35,7 @@
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
+#include "camel-mapi-settings.h"
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-summary.h"
 #include "camel-mapi-notifications.h"
@@ -42,9 +43,6 @@
 #include <exchange-mapi-utils.h>
 //#define d(x) x
 
-/* This definition should be in-sync with those in exchange-mapi-account-setup.c and exchange-account-listener.c */
-#define E_PASSWORD_COMPONENT "ExchangeMAPI"
-
 #include <sys/types.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -55,11 +53,9 @@
 #define d(x) printf("%s:%s:%s \n", G_STRLOC, G_STRFUNC, x)
 
 struct _CamelMapiStorePrivate {
-	gchar *profile;
 	ExchangeMapiConnection *conn;
 
 	gchar *base_url;
-	gchar *storage_path;
 
 	GHashTable *id_hash; /*get names from ids*/
 	GHashTable *name_hash;/*get ids from names*/
@@ -71,12 +67,19 @@ struct _CamelMapiStorePrivate {
 	gpointer notification_data; /* pointer to a notification data; can be only one */
 };
 
-G_DEFINE_TYPE (CamelMapiStore, camel_mapi_store, CAMEL_TYPE_OFFLINE_STORE)
+/* Forward Declarations */
+static void camel_subscribable_init (CamelSubscribableInterface *interface);
+
+G_DEFINE_TYPE_WITH_CODE (
+	CamelMapiStore,
+	camel_mapi_store,
+	CAMEL_TYPE_OFFLINE_STORE,
+	G_IMPLEMENT_INTERFACE (
+		CAMEL_TYPE_SUBSCRIBABLE,
+		camel_subscribable_init))
 
 /* service methods */
-static gboolean	mapi_construct(CamelService *, CamelSession *,
-				     CamelProvider *, CamelURL *,
-				     GError **);
+static void mapi_store_constructed (GObject *object);
 static gchar	*mapi_get_name(CamelService *, gboolean );
 static gboolean	mapi_connect_sync(CamelService *, GCancellable *cancellable, GError **);
 static gboolean	mapi_disconnect_sync(CamelService *, gboolean , GCancellable *cancellable, GError **);
@@ -256,7 +259,9 @@ mapi_get_folder_with_type (CamelStore *store, guint folder_type, GCancellable *c
 }
 
 static CamelFolderInfo *
-mapi_convert_to_folder_info (CamelMapiStore *store, ExchangeMAPIFolder *folder, const gchar *url, GError **error)
+mapi_convert_to_folder_info (CamelMapiStore *store,
+                             ExchangeMAPIFolder *folder,
+                             GError **error)
 {
 	gchar *name;
 	gchar *parent, *id = NULL;
@@ -313,17 +318,15 @@ mapi_convert_to_folder_info (CamelMapiStore *store, ExchangeMAPIFolder *folder,
 	mapi_id_folder = exchange_mapi_folder_get_parent_id (folder);
 	parent = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", mapi_id_folder);
 
-	fi->name = name;
+	fi->display_name = name;
 
 	par_name = mapi_folders_hash_table_name_lookup (store, parent, TRUE);
 	if (par_name != NULL) {
 		gchar *str = g_strconcat (par_name, "/", name, NULL);
 
 		fi->full_name = str; /* takes ownership of the string */
-		fi->uri = g_strconcat (url, str, NULL);
 	} else {
 		fi->full_name = g_strdup (name);
-		fi->uri = g_strconcat (url, "", name, NULL);
 	}
 
 	/*name_hash returns the container id given the name */
@@ -361,16 +364,15 @@ remove_path_from_store_summary (const gchar *path, gpointer value, CamelMapiStor
 		fi = camel_folder_info_new ();
 		fi->unread = -1;
 		fi->total = -1;
-		fi->uri = g_strdup (camel_store_info_uri (mstore->summary, si));
-		fi->name = g_strdup (camel_store_info_name (mstore->summary, si));
+		fi->display_name = g_strdup (camel_store_info_name (mstore->summary, si));
 		fi->full_name = g_strdup (camel_mapi_store_info_full_name (mstore->summary, si));
-		if (!fi->name && fi->full_name) {
-			fi->name = strrchr (fi->full_name, '/');
-			if (fi->name)
-				fi->name = g_strdup (fi->name + 1);
+		if (!fi->display_name && fi->full_name) {
+			fi->display_name = strrchr (fi->full_name, '/');
+			if (fi->display_name)
+				fi->display_name = g_strdup (fi->display_name + 1);
 		}
 
-		camel_store_folder_unsubscribed (CAMEL_STORE (mstore), fi);
+		camel_subscribable_folder_unsubscribed (CAMEL_SUBSCRIBABLE (mstore), fi);
 		camel_store_folder_deleted (CAMEL_STORE (mstore), fi);
 		camel_folder_info_free (fi);
 
@@ -386,7 +388,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	CamelMapiStorePrivate  *priv = store->priv;
 	gboolean status;
 	GSList *folder_list = NULL, *temp_list = NULL, *list = NULL;
-	gchar *url, *temp_url;
 	gboolean subscription_list = FALSE;
 	CamelFolderInfo *info = NULL;
 	CamelMapiStoreInfo *mapi_si = NULL;
@@ -445,16 +446,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	temp_list = folder_list;
 	list = folder_list;
 
-	url = camel_url_to_string (CAMEL_SERVICE(store)->url,
-				   (CAMEL_URL_HIDE_PASSWORD|
-				    CAMEL_URL_HIDE_PARAMS|
-				    CAMEL_URL_HIDE_AUTH));
-	if ( url[strlen(url) - 1] != '/') {
-		temp_url = g_strconcat (url, "/", NULL);
-		g_free ((gchar *)url);
-		url = temp_url;
-	}
-
 	/*populate the hash table for finding the mapping from container id <-> folder name*/
 	for (;temp_list != NULL; temp_list = g_slist_next (temp_list) ) {
 		const gchar *full_name = NULL;
@@ -506,7 +497,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			continue;
 
 		if ( folder->container_class == MAPI_FOLDER_TYPE_MAIL) {
-			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, NULL);
+			info = mapi_convert_to_folder_info (store, folder, NULL);
 			info->flags |= CAMEL_MAPI_FOLDER_MAIL;
 			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
 
@@ -528,7 +519,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 
 				if (!subscription_list) {
 					camel_store_folder_created (CAMEL_STORE (store), info);
-					camel_store_folder_subscribed (CAMEL_STORE (store), info);
+					camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (store), info);
 				}
 			}
 
@@ -540,7 +531,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			camel_folder_info_free (info);
 		} else if (folder->category == MAPI_FAVOURITE_FOLDER) {
 			gchar *fid, *pfid = NULL;
-			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, NULL);
+			info = mapi_convert_to_folder_info (store, folder, NULL);
 			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
 			fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
 						exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
@@ -567,8 +558,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	camel_store_summary_touch ((CamelStoreSummary *)store->summary);
 	camel_store_summary_save ((CamelStoreSummary *)store->summary);
 
-	g_free (url);
-
 	g_slist_foreach (list, (GFunc) exchange_mapi_folder_free, NULL);
 	g_slist_free (list);
 
@@ -630,12 +619,12 @@ static void
 fix_folder_names (CamelFolderInfo *fi)
 {
 	while (fi) {
-		if (fi->name && strchr (fi->name, '\\')) {
+		if (fi->display_name && strchr (fi->display_name, '\\')) {
 			gchar *unescaped;
 
-			unescaped = unescape_slash (fi->name);
-			g_free (fi->name);
-			fi->name = unescaped;
+			unescaped = unescape_slash (fi->display_name);
+			g_free (fi->display_name);
+			fi->display_name = unescaped;
 		}
 
 		if (fi->child)
@@ -654,7 +643,7 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 	GPtrArray *folders;
 	gchar *path, *name;
 	gint i;
-	gboolean subscribed, favourites = FALSE, subscription_list = FALSE;
+	gboolean subscribed, subscription_list = FALSE;
 
 	subscription_list = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST);
 	subscribed = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
@@ -710,20 +699,9 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 				folder_name = strrchr(store_info_path,'/');
 				if (folder_name != NULL)
 					store_info_path = ++folder_name;
-
-				favourites = TRUE;
 			}
 
 			fi = mapi_build_folder_info(mapi_store, parent_name, store_info_path);
-			if (favourites) {
-				CamelURL *url;
-				url = camel_url_new(mapi_store->priv->base_url,NULL);
-				url->path = g_strdup_printf("/%s", camel_store_info_path((CamelStoreSummary *)mapi_store->summary, si));
-				g_free (fi->uri);
-				fi->uri = camel_url_to_string(url,CAMEL_URL_HIDE_ALL);
-				camel_url_free (url);
-			}
-
 			fi->unread = si->unread;
 			fi->total = si->total;
 			fi->flags = si->flags;
@@ -754,15 +732,16 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 static gboolean
 mapi_forget_folder (CamelMapiStore *mapi_store, const gchar *folder_name, GError **error)
 {
-	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
+	const gchar *user_data_dir;
 	gchar *state_file;
 	gchar *folder_dir, *storage_path;
 	CamelFolderInfo *fi;
-	const gchar *name;
 
-	name = folder_name;
+	service = CAMEL_SERVICE (mapi_store);
+	user_data_dir = camel_service_get_user_data_dir (service);
 
-	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
+	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
 
 	/* Fixme Path - e_*-to_path */
 	folder_dir = g_strconcat (storage_path, "/", folder_name, NULL);
@@ -862,8 +841,6 @@ mapi_store_finalize (GObject *object)
 
 	priv = CAMEL_MAPI_STORE (object)->priv;
 
-	g_free (priv->profile);
-	g_free (priv->storage_path);
 	g_free (priv->base_url);
 
 	if (priv->id_hash != NULL)
@@ -905,28 +882,20 @@ mapi_store_can_refresh_folder (CamelStore *store,
                                CamelFolderInfo *info,
                                GError **error)
 {
+	CamelService *service;
+	CamelSettings *settings;
+	gboolean check_all;
+
 	/* skip unselectable folders from automatic refresh */
 	if (info && (info->flags & CAMEL_FOLDER_NOSELECT) != 0)
 		return FALSE;
 
-	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, error) ||
-	      (camel_url_get_param (((CamelService *)store)->url, "check_all") != NULL);
-}
+	service = CAMEL_SERVICE (store);
+	settings = camel_service_get_settings (service);
 
-static gboolean
-mapi_store_folder_is_subscribed (CamelStore *store,
-                                 const gchar *folder_name)
-{
-	CamelMapiStore *mapi_store = (CamelMapiStore *) store;
-	CamelStoreInfo *si;
-	gint truth = FALSE;
+	check_all = camel_mapi_settings_get_check_all (CAMEL_MAPI_SETTINGS (settings));
 
-	if ((si = camel_store_summary_path ((CamelStoreSummary *) mapi_store->summary, folder_name))) {
-		truth = (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) != 0;
-		camel_store_summary_info_free ((CamelStoreSummary *) mapi_store->summary, si);
-	}
-
-	return truth;
+	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, error) || check_all;
 }
 
 static CamelFolder *
@@ -937,9 +906,10 @@ mapi_store_get_folder_sync (CamelStore *store,
                             GError **error)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
 	CamelStoreInfo *si;
 	CamelFolder *folder;
+	const gchar *user_data_dir;
 	gchar *storage_path;
 
 	si = camel_mapi_store_summary_full_name (mapi_store->summary, folder_name);
@@ -970,7 +940,10 @@ mapi_store_get_folder_sync (CamelStore *store,
 	if (si)
 		camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 
-	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
+	service = CAMEL_SERVICE (store);
+	user_data_dir = camel_service_get_user_data_dir (service);
+
+	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
 	folder = camel_mapi_folder_new (store, folder_name, storage_path, flags, error);
 	g_free (storage_path);
 
@@ -985,30 +958,34 @@ mapi_store_get_folder_info_sync (CamelStore *store,
                                  GError **error)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelServiceConnectionStatus status;
+	CamelService *service;
 
 	/*
 	 * Thanks to Michael, for his cached folders implementation in IMAP
 	 * is used as is here.
 	 */
 
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	service = CAMEL_SERVICE (store);
+	status = camel_service_get_connection_status (service);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))) {
-		if (((CamelService *)store)->status == CAMEL_SERVICE_DISCONNECTED) {
-			((CamelService *)store)->status = CAMEL_SERVICE_CONNECTING;
-			mapi_connect_sync ((CamelService *)store, cancellable, NULL);
-		}
+		if (status == CAMEL_SERVICE_DISCONNECTED)
+			camel_service_connect_sync (service, NULL);
 
 		/* update folders from the server only when asking for the top most or the 'top' is not known;
 		   otherwise believe the local cache, because folders sync is pretty slow operation to be done
 		   one every single question on the folder info */
+		status = camel_service_get_connection_status (service);
 		if (((flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST) != 0 ||
 		    (!(flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED)) ||
 		    (!mapi_store->priv->folders_synced) ||
 		    (top && *top && !camel_mapi_store_folder_id_lookup (mapi_store, top))) &&
-		    (check_for_connection ((CamelService *)store, NULL) || ((CamelService *)store)->status == CAMEL_SERVICE_CONNECTING)) {
+		    (check_for_connection (service, NULL) || status == CAMEL_SERVICE_CONNECTING)) {
 			if (!mapi_folders_sync (mapi_store, flags, error)) {
-				camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+				camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 				return NULL;
 			}
 			camel_store_summary_touch ((CamelStoreSummary *)mapi_store->summary);
@@ -1016,7 +993,7 @@ mapi_store_get_folder_info_sync (CamelStore *store,
 		}
 	}
 
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return mapi_get_folder_info_offline (store, top, flags, error);
 }
@@ -1096,7 +1073,7 @@ mapi_store_create_folder_sync (CamelStore *store,
 		mapi_update_folder_hash_tables (mapi_store, root->full_name, fid, parent_id);
 
 		camel_store_folder_created (store, root);
-		camel_store_folder_subscribed (store, root);
+		camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (store), root);
 	} else {
 		if (mapi_error) {
 			g_set_error (
@@ -1190,16 +1167,21 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelMapiStorePrivate  *priv = mapi_store->priv;
 	CamelStoreInfo *si = NULL;
+	CamelService *service;
+	const gchar *user_data_dir;
 	gchar *old_parent, *new_parent, *tmp;
 	gboolean move_cache = TRUE;
 	const gchar *old_fid_str, *new_parent_fid_str = NULL;
 	mapi_id_t old_fid;
 	GError *local_error = NULL;
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	service = CAMEL_SERVICE (store);
+	user_data_dir = camel_service_get_user_data_dir (service);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (!camel_mapi_store_connected ((CamelMapiStore *)store, &local_error)) {
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 		if (local_error != NULL) {
 			g_propagate_error (error, local_error);
@@ -1217,7 +1199,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI folder '%s'. Folder does not exist."),
 			old_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
@@ -1229,7 +1211,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI default folder '%s' to '%s'."),
 			old_name, new_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
@@ -1256,7 +1238,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI folder '%s' to '%s'"),
 			old_name, new_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		g_free (old_parent);
 		g_free (new_parent);
 		return FALSE;
@@ -1283,7 +1265,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 					old_name, new_name);
 			}
 
-			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 			g_free (old_parent);
 			g_free (new_parent);
 			return FALSE;
@@ -1331,7 +1313,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			   !exchange_mapi_util_mapi_id_from_string (old_parent_fid_str, &old_parent_fid) ||
 			   !exchange_mapi_util_mapi_id_from_string (new_parent_fid_str, &new_parent_fid) ||
 			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, 0, new_parent_fid, 0, tmp, &local_error)) {
-			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 			if (local_error) {
 				g_set_error (
 					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
@@ -1365,7 +1347,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 		g_free (folder_id);
 	}
 
-	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	si = camel_store_summary_path ((CamelStoreSummary *)mapi_store->summary, old_name);
 	if (si) {
@@ -1381,8 +1363,8 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	if (move_cache) {
 		gchar *oldpath, *newpath;
 
-		oldpath = g_build_filename (priv->storage_path, "folders", old_name, NULL);
-		newpath = g_build_filename (priv->storage_path, "folders", new_name, NULL);
+		oldpath = g_build_filename (user_data_dir, "folders", old_name, NULL);
+		newpath = g_build_filename (user_data_dir, "folders", new_name, NULL);
 
 		if (g_file_test (oldpath, G_FILE_TEST_IS_DIR) && g_rename (oldpath, newpath) == -1 && errno != ENOENT) {
 			g_warning ("Could not rename message cache '%s' to '%s': %s: cache reset", oldpath, newpath, g_strerror (errno));
@@ -1399,16 +1381,39 @@ mapi_store_rename_folder_sync (CamelStore *store,
 }
 
 static gboolean
-mapi_store_subscribe_folder_sync (CamelStore *store,
+mapi_store_noop_sync (CamelStore *store,
+                      GCancellable *cancellable,
+                      GError **error)
+{
+	return TRUE;
+}
+
+static gboolean
+mapi_store_folder_is_subscribed (CamelSubscribable *subscribable,
+                                 const gchar *folder_name)
+{
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
+	CamelStoreInfo *si;
+	gint truth = FALSE;
+
+	if ((si = camel_store_summary_path ((CamelStoreSummary *) mapi_store->summary, folder_name))) {
+		truth = (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) != 0;
+		camel_store_summary_info_free ((CamelStoreSummary *) mapi_store->summary, si);
+	}
+
+	return truth;
+}
+
+static gboolean
+mapi_store_subscribe_folder_sync (CamelSubscribable *subscribable,
                                   const gchar *folder_name,
                                   GCancellable *cancellable,
                                   GError **error)
 {
-	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si = NULL;
 	const gchar *parent_name = NULL, *use_folder_name = folder_name, *fid = NULL;
-	gboolean favourites = FALSE;
 	/* TODO : exchange_mapi_add_to_favorites (); */
 
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
@@ -1425,7 +1430,6 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 		}
 
 		use_folder_name = ++f_name;
-		favourites = TRUE;
 	}
 
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
@@ -1446,33 +1450,27 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 
 	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
 		fi = mapi_build_folder_info (mapi_store, parent_name, use_folder_name);
-		if (favourites) {
-			CamelURL *url;
-			url = camel_url_new (mapi_store->priv->base_url, NULL);
-			url->path = g_strdup_printf ("/%s", camel_store_info_path (mapi_store->summary, si));
-			g_free (fi->uri);
-			fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
-			camel_url_free (url);
-		}
-
 		fi->unread = si->unread;
 		fi->total = si->total;
 		fi->flags = si->flags;
 		fi->flags |= CAMEL_FOLDER_SUBSCRIBED;
 		fi->flags |= CAMEL_FOLDER_NOCHILDREN;
 		fi->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
-		camel_store_folder_subscribed (store, fi);
+		camel_subscribable_folder_subscribed (subscribable, fi);
 		camel_folder_info_free (fi);
 	} else {
+		CamelURL *url;
 		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, folder_name);
-		exchange_mapi_add_esource (CAMEL_SERVICE(mapi_store)->url, use_folder_name, fid, folder_type);
+
+		url = camel_service_get_camel_url (CAMEL_SERVICE (mapi_store));
+		exchange_mapi_add_esource (url, use_folder_name, fid, folder_type);
 	}
 	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	return TRUE;
 }
 
 static gboolean
-mapi_store_unsubscribe_folder_sync (CamelStore *store,
+mapi_store_unsubscribe_folder_sync (CamelSubscribable *subscribable,
                                     const gchar *folder_name,
                                     GCancellable *cancellable,
                                     GError **error)
@@ -1483,8 +1481,10 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	const gchar *fid = NULL, *use_folder_name = folder_name;
 	gchar *f_name = NULL;
 
-	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-	CamelURL *url = CAMEL_SERVICE (mapi_store)->url;
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (CAMEL_SERVICE (mapi_store));
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
 	if (si) {
@@ -1509,7 +1509,7 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	}
 	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
 		fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
-		camel_store_folder_unsubscribed (store, fi);
+		camel_subscribable_folder_unsubscribed (subscribable, fi);
 		camel_folder_info_free (fi);
 	} else {
 		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, use_folder_name);
@@ -1521,14 +1521,6 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	return TRUE;
 }
 
-static gboolean
-mapi_store_noop_sync (CamelStore *store,
-                      GCancellable *cancellable,
-                      GError **error)
-{
-	return TRUE;
-}
-
 static void
 camel_mapi_store_class_init (CamelMapiStoreClass *class)
 {
@@ -1541,9 +1533,10 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	object_class = G_OBJECT_CLASS (class);
 	object_class->dispose = mapi_store_dispose;
 	object_class->finalize = mapi_store_finalize;
+	object_class->constructed = mapi_store_constructed;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
-	service_class->construct = mapi_construct;
+	service_class->settings_type = CAMEL_TYPE_MAPI_SETTINGS;
 	service_class->get_name = mapi_get_name;
 	service_class->connect_sync = mapi_connect_sync;
 	service_class->disconnect_sync = mapi_disconnect_sync;
@@ -1553,7 +1546,6 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	store_class->hash_folder_name = mapi_store_hash_folder_name;
 	store_class->compare_folder_name = mapi_store_compare_folder_name;
 	store_class->can_refresh_folder = mapi_store_can_refresh_folder;
-	store_class->folder_is_subscribed = mapi_store_folder_is_subscribed;
 	store_class->free_folder_info = camel_store_free_folder_info_full;
 	store_class->get_folder_sync = mapi_store_get_folder_sync;
 	store_class->get_folder_info_sync = mapi_store_get_folder_info_sync;
@@ -1562,11 +1554,17 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	store_class->create_folder_sync = mapi_store_create_folder_sync;
 	store_class->delete_folder_sync = mapi_store_delete_folder_sync;
 	store_class->rename_folder_sync = mapi_store_rename_folder_sync;
-	store_class->subscribe_folder_sync = mapi_store_subscribe_folder_sync;
-	store_class->unsubscribe_folder_sync = mapi_store_unsubscribe_folder_sync;
 	store_class->noop_sync = mapi_store_noop_sync;
 }
 
+static void
+camel_subscribable_init (CamelSubscribableInterface *interface)
+{
+	interface->folder_is_subscribed = mapi_store_folder_is_subscribed;
+	interface->subscribe_folder_sync = mapi_store_subscribe_folder_sync;
+	interface->unsubscribe_folder_sync = mapi_store_unsubscribe_folder_sync;
+}
+
 /*
 ** store is already initilyse to NULL or 0 value
 ** class already have a parent_class
@@ -1579,26 +1577,25 @@ camel_mapi_store_init (CamelMapiStore *mapi_store)
 }
 
 /* service methods */
-static gboolean mapi_construct(CamelService *service, CamelSession *session,
-				 CamelProvider *provider, CamelURL *url,
-				 GError **error)
+static void
+mapi_store_constructed (GObject *object)
 {
-	CamelMapiStore	*mapi_store = CAMEL_MAPI_STORE (service);
-	CamelStore *store = CAMEL_STORE (service);
+	CamelMapiStore	*mapi_store = CAMEL_MAPI_STORE (object);
+	CamelStore *store = CAMEL_STORE (object);
 	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
+	const gchar *user_data_dir;
 	gchar *path = NULL;
 
-	if (!CAMEL_SERVICE_CLASS (camel_mapi_store_parent_class)->construct (service, session, provider, url, error))
-		return FALSE;
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (camel_mapi_store_parent_class)->constructed (object);
 
-	/*storage path*/
-	priv->storage_path = camel_session_get_storage_path (session, service, error);
-	if (!priv->storage_path)
-		return FALSE;
+	service = CAMEL_SERVICE (object);
+	user_data_dir = camel_service_get_user_data_dir (service);
 
 	/*store summary*/
-	path = g_alloca (strlen (priv->storage_path) + 32);
-	sprintf (path, "%s/.summary", priv->storage_path);
+	path = g_alloca (strlen (user_data_dir) + 32);
+	sprintf (path, "%s/.summary", user_data_dir);
 
 	mapi_store->summary = camel_mapi_store_summary_new ();
 	camel_store_summary_set_filename ((CamelStoreSummary *)mapi_store->summary, path);
@@ -1606,17 +1603,12 @@ static gboolean mapi_construct(CamelService *service, CamelSession *session,
 	camel_store_summary_touch ((CamelStoreSummary *)mapi_store->summary);
 	camel_store_summary_load ((CamelStoreSummary *) mapi_store->summary);
 
-	/*user and profile*/
-	priv->profile = g_strdup (camel_url_get_param(url, "profile"));
-
 	/*base url*/
-	priv->base_url = camel_url_to_string (service->url, (CAMEL_URL_HIDE_PASSWORD |
-						       CAMEL_URL_HIDE_PARAMS   |
-						       CAMEL_URL_HIDE_AUTH)  );
-
-	/*filter*/
-	if (camel_url_get_param (url, "filter"))
-		store->flags |= CAMEL_STORE_FILTER_INBOX;
+	priv->base_url = camel_url_to_string (
+		camel_service_get_camel_url (service),
+		CAMEL_URL_HIDE_PASSWORD |
+		CAMEL_URL_HIDE_PARAMS |
+		CAMEL_URL_HIDE_AUTH);
 
 	/*Hash Table*/
 	priv->id_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); /* folder ID to folder Full name */
@@ -1627,75 +1619,110 @@ static gboolean mapi_construct(CamelService *service, CamelSession *session,
 	store->flags &= ~CAMEL_STORE_VJUNK;
 	store->flags &= ~CAMEL_STORE_VTRASH;
 
-	store->flags |= CAMEL_STORE_SUBSCRIPTIONS | CAMEL_STORE_REAL_JUNK_FOLDER;
-
-	return TRUE;
+	store->flags |= CAMEL_STORE_REAL_JUNK_FOLDER;
 }
 
-static char
-*mapi_get_name(CamelService *service, gboolean brief)
+static char *
+mapi_get_name(CamelService *service, gboolean brief)
 {
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (service);
+
 	if (brief) {
 		/* Translators: The %s is replaced with a server's host name */
-		return g_strdup_printf(_("Exchange MAPI server %s"), service->url->host);
+		return g_strdup_printf(_("Exchange MAPI server %s"), url->host);
 	} else {
 		/*To translators : Example string : Exchange MAPI service for
 		  _username_ on _server host name__*/
 		return g_strdup_printf(_("Exchange MAPI service for %s on %s"),
-				       service->url->user, service->url->host);
+				       url->user, url->host);
 	}
 }
 
+static gboolean
+mapi_prompt_pass_creds (CamelService *service, CamelURL *url, const gchar *reason, GError **error) {
+	gchar *prompt;
+	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
+	CamelSession *session = camel_service_get_session (service);
+
+	if (reason) {
+		/* We need to un-cache the password before prompting again */
+		prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
+		camel_url_set_passwd (url, NULL);
+	}
+
+	/*To translators : First %s : is the error text or the reason
+	  for prompting the user if it is available.
+	 Second %s is : Username.
+	 Third %s is : Server host name.*/
+	prompt = g_strdup_printf (_("%s Please enter the MAPI password for %s@%s"), reason ? reason : "", url->user, url->host);
+	url->passwd = camel_session_get_password (session, service, prompt, "password", prompt_flags, NULL);
+	g_free (prompt);
+
+	if (!url->passwd) {
+		if (error && !*error)
+			g_set_error (
+				error, G_IO_ERROR,
+				G_IO_ERROR_CANCELLED,
+				_("You did not enter a password."));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 mapi_auth_loop (CamelService *service, GError **error)
 {
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
-	CamelSession *session = camel_service_get_session (service);
+	CamelURL *url;
+	CamelSettings *settings;
+	CamelMapiSettings *mapi_settings;
+	ExchangeMapiProfileData empd = { 0 };
 
 	gchar *errbuf = NULL;
-	gboolean authenticated = FALSE;
-	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
+	gboolean authenticated = FALSE, ret = TRUE, krb_requested = FALSE;
+	const gchar *profile;
 
-	service->url->passwd = NULL;
+	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
+	mapi_settings = CAMEL_MAPI_SETTINGS (settings);
+
+	empd.server = url->host;
+	empd.username = url->user;
+	exchange_mapi_util_profiledata_from_settings (&empd, mapi_settings);
+
+	profile = camel_mapi_settings_get_profile (mapi_settings);
+
+	url->passwd = NULL;
 
 	while (!authenticated) {
 		GError *mapi_error = NULL;
 
-		if (errbuf) {
-			/* We need to un-cache the password before prompting again */
-			prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-			g_free (service->url->passwd);
-			service->url->passwd = NULL;
-		}
+		if (!url->passwd || errbuf) {
+			const gchar *why = errbuf ? errbuf : NULL;
 
-		if (!service->url->passwd ) {
-			gchar *prompt;
+			if (empd.krb_sso) {
+				if (!krb_requested) {
+					ret = exchange_mapi_util_trigger_krb_auth (&empd, error);
+					krb_requested = TRUE;
+				} else {
+					ret = FALSE;
+				}
+			} else {
+				ret = mapi_prompt_pass_creds (service, url, why, error);
+			}
 
-			/*To translators : First %s : is the error text or the reason
-			  for prompting the user if it is available.
-			 Second %s is : Username.
-			 Third %s is : Server host name.*/
-			prompt = g_strdup_printf (_("%s Please enter the MAPI password for %s@%s"),
-						  errbuf ? errbuf : "",
-						  service->url->user,
-						  service->url->host);
-			service->url->passwd =
-				camel_session_get_password (session, service, E_PASSWORD_COMPONENT,
-							    prompt, "password", prompt_flags, NULL);
-			g_free (prompt);
 			g_free (errbuf);
 			errbuf = NULL;
-
-			if (!service->url->passwd) {
-				g_set_error (
-					error, G_IO_ERROR,
-					G_IO_ERROR_CANCELLED,
-					_("You did not enter a password."));
-				return FALSE;
-			}
 		}
 
-		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, service->url->passwd, &mapi_error);
+		if (!ret || (error && *error)) {
+			return FALSE;
+		}
+
+		store->priv->conn = exchange_mapi_connection_new (profile, url->passwd, &mapi_error);
 		if (!store->priv->conn || !exchange_mapi_connection_connected (store->priv->conn)) {
 			if (mapi_error) {
 				errbuf = g_strdup_printf (_("Unable to authenticate to Exchange MAPI server: %s"), mapi_error->message);
@@ -1716,20 +1743,14 @@ mapi_connect_sync (CamelService *service,
                    GError **error)
 {
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
-	CamelMapiStorePrivate *priv = store->priv;
+	CamelServiceConnectionStatus status;
 	guint16 event_mask = 0;
 
-	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))) {
+	status = camel_service_get_connection_status (service);
+	if (status == CAMEL_SERVICE_DISCONNECTED) {
 		return FALSE;
 	}
 
-	if (!priv) {
-		store->priv = g_new0 (CamelMapiStorePrivate, 1);
-		priv = store->priv;
-		if (!camel_service_construct (service, service->session, service->provider, service->url, error))
-			return FALSE;
-	}
-
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	if (check_for_connection (service, NULL)) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -1742,7 +1763,6 @@ mapi_connect_sync (CamelService *service,
 		return FALSE;
 	}
 
-	service->status = CAMEL_SERVICE_CONNECTED;
 	camel_offline_store_set_online_sync (
 		CAMEL_OFFLINE_STORE (store), TRUE, cancellable, NULL);
 
@@ -1791,7 +1811,6 @@ mapi_disconnect_sync (CamelService *service,
 	}
 
 	store->priv->folders_synced = FALSE;
-	service->status = CAMEL_SERVICE_DISCONNECTED;
 
 	return TRUE;
 }
@@ -1913,10 +1932,8 @@ camel_mapi_store_summary_path_to_full(CamelMapiStoreSummary *s, const gchar *pat
 static CamelFolderInfo *
 mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, const gchar *folder_name)
 {
-	CamelURL *url;
 	const gchar *name;
 	CamelFolderInfo *fi;
-	CamelMapiStorePrivate *priv = mapi_store->priv;
 
 	fi = camel_folder_info_new ();
 
@@ -1928,20 +1945,13 @@ mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, con
 	else
 		fi->full_name = g_strdup (folder_name);
 
-	url = camel_url_new (priv->base_url, NULL);
-	g_free(url->path);
-
-	url->path = g_strdup_printf("/%s", fi->full_name);
-	fi->uri = camel_url_to_string(url,CAMEL_URL_HIDE_ALL);
-	camel_url_free(url);
-
 	name = strrchr(fi->full_name,'/');
 	if (name == NULL)
 		name = fi->full_name;
 	else
 		name++;
 
-	fi->name = g_strdup (name);
+	fi->display_name = g_strdup (name);
 
 	return fi;
 }
@@ -2086,18 +2096,6 @@ camel_mapi_store_folder_lookup (CamelMapiStore *mapi_store, const gchar *folder_
 	return g_hash_table_lookup (priv->id_hash, folder_id);
 }
 
-const gchar *
-camel_mapi_store_get_profile_name (CamelMapiStore *mapi_store)
-{
-	CamelMapiStorePrivate *priv;
-
-	g_return_val_if_fail (CAMEL_IS_MAPI_STORE (mapi_store), NULL);
-
-	priv = mapi_store->priv;
-
-	return priv->profile;
-}
-
 ExchangeMapiConnection *
 camel_mapi_store_get_exchange_connection (CamelMapiStore *mapi_store)
 {
